#!/usr/bin/python3
"""
## © University of Southampton IT Innovation Centre, 2018
##
## Copyright in this software belongs to University of Southampton
## IT Innovation Centre of Gamma House, Enterprise Road,
## Chilworth Science Park, Southampton, SO16 7NS, UK.
##
## This software may not be used, sold, licensed, transferred, copied
## or reproduced in whole or in part in any manner or form or in or
## on any media by any person other than in accordance with the terms
## of the Licence Agreement supplied with the software, or otherwise
## without the prior written consent of the copyright owners.
##
## This software is distributed WITHOUT ANY WARRANTY, without even the
## implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE, except where stated in the Licence Agreement supplied with
## the software.
##
##      Created By :            Nikolay Stanchev
##      Created Date :          17-04-2018
##      Created for Project :   FLAME
"""

import pytest
import time


class TestE2ESimulation(object):
    """
    A testing class used to group all the tests related to the E2E simulation data
    """

    @pytest.fixture(scope='class', autouse=True)
    def run_simulator(self, e2e_simulator):
        print("Running simulation, please wait...")
        e2e_simulator.run()

        print("Waiting for INFLUX to finish receiving simulation data...")
        time.sleep(5)  # wait for data to finish arriving at the INFLUX database
        print("... simulation data fixture finished")

    @pytest.mark.parametrize("query, expected_result", [
        ('SELECT count(*) FROM "E2EMetrics"."autogen"."net"',
         {"time": "1970-01-01T00:00:00Z", "count_delay": 11}),
        ('SELECT count(*) FROM "E2EMetrics"."autogen"."service"',
         {"time": "1970-01-01T00:00:00Z", "count_response_time": 5}),
        ('SELECT count(*) FROM "E2EMetrics"."autogen"."e2e"',
         {"time": "1970-01-01T00:00:00Z", "count_Dnet": 11, "count_Dresponse_time": 11}),
        ])
    def test_simulation(self, influx_db, query, expected_result):
        """
        This is the entry point of the test. This method will be found and executed when the module is ran using pytest

        :param query: the query to execute (value obtained from the pytest parameter decorator)
        :param expected_result: the result expected from executing the query (value obtained from the pytest parameter decorator)
        :param influx_db the import db client fixture - imported from contest.py
        """

        # pytest automatically goes through all queries under test, declared in the parameters decorator
        print("\n")  # prints a blank line for formatting purposes

        # the raise_errors=False argument is given so that we could actually test that the DB didn't return any errors instead of raising an exception
        query_result = influx_db.query(query, raise_errors=False)

        # test the error attribute of the result is None, that is no error is returned from executing the DB query
        assert query_result.error is None, "An error was encountered while executing query {0}.".format(query)

        # get the dictionary of result points; the next() function just gets the first element of the query results generator (we only expect one item in the generator)
        actual_result = next(query_result.get_points())

        assert expected_result == actual_result, "E2E Simulation test failure"

        print("Successfully passed test for the following query: {0}".format(query))
